#/*----------------------------   problem.h     ---------------------------*/
/*      $Id: problem.h,v 1.2 2009/10/26 09:07:16 richter Exp $                 */
#ifndef __problem_H
#define __problem_H
/*----------------------------   problem.h     ---------------------------*/

#include  "problemdescriptorbase.h"
#include  "myequation.h"
#include  "dirichletdata.h"
#include  "domainrighthandside.h"
#include  "periodicdata.h"
#include<fstream>
#include  "zerodirichletdata.h"
#include "componentinformationbase.h"

extern double TIME, DT,DTSUB,DDD;

using namespace std;

namespace Gascoigne
{

  // class for specifying Dirichlet data
  class SeaIceDirichletData : public DirichletData
  {
  public:
    SeaIceDirichletData(const ParamFile& pf) : DirichletData(pf)
    {}
    std::string GetName() const { return "SeaIce Dirichlet Data"; }
    
    void operator()(DoubleVector& b, const Vertex2d& v, int col) const
    {
      b.zero();
    }
  };
  // initialisation of indicator function
  class PhiDirichletData : public DirichletData
  {
  public:
    PhiDirichletData(const ParamFile& pf) : DirichletData(pf)
    {}
    std::string GetName() const { return "Phi Dirichlet Data"; }
    
    void operator()(DoubleVector& b, const Vertex2d& v, int col) const
    {
    
     // sea ice cover
     b[0] = 0.0;
     b[1] = 0.0;
     
     // iceberg
    if (v.x() > 0.0015 && v.x() < 0.0035 && v.y() > 0.002 && v.y() < 0.003) {
       b[0] = 0.0;
       b[1] = 0.0;
     }
    }
  };
  // initialisation of H and A
  class TGDirichletData : public DirichletData
  {
  public:
    TGDirichletData(const ParamFile& pf) : DirichletData(pf)
    {}
    std::string GetName() const { return "TG Dirichlet Data"; }
    
    void operator()(DoubleVector& b, const Vertex2d& v, int col) const
    {
    
     // sea ice cover
     b[0] = 0.1; 
     b[1] = 0.1; 
    
     // icebergs
     if (v.x() > 0.0015 && v.x() < 0.0035 && v.y() > 0.002 && v.y() < 0.003) { 
       b[0] = 10.0; // thickness
       b[1] = 1.0; // concentration
     }

    }
  };

  /*---------------------------------------------------*/
  // right hand side for phi 
  class PhiRhs : public virtual DomainRightHandSide
  {
    SeaIceData data;
    
    mutable FemFunction *oldh, *vel, *oldphi;

    void SetFemData(FemData& q) const 
    { 
      assert(q.find("oldh")!=q.end()); 
      oldh = &q["oldh"];

      assert(q.find("vel")!=q.end()); 
      vel = &q["vel"];

      assert(q.find("oldphi")!=q.end());
      oldphi = &q["oldphi"];
    }

  public:
    
  PhiRhs(const SeaIceData& dat) : DomainRightHandSide(), data(dat)
      {
      }
  PhiRhs* createNew() const { return new PhiRhs(data); } 

    std::string GetName() const {return "Phi Rhs";}    
    int GetNcomp() const {return 2; }
    
    void operator()(VectorIterator b, const TestFunction& N, const Vertex2d& v) const 
    {
      // equation for phi
      b[0] += (*oldphi)[0].m() * N.m() 
	- DTSUB *( (*vel)[0].x() + (*vel)[1].y()) * (*oldphi)[0].m() * N.m() 
	- DTSUB * ((*vel)[0].m() * (*oldphi)[0].x() + (*vel)[1].m()*(*oldphi)[0].y()) * N.m() 
	- 0.5 * DTSUB * DTSUB * ((*vel)[0].m() * (*oldphi)[0].x() + (*vel)[1].m()*(*oldphi)[0].y()) 
	* ((*vel)[0].m() * N.x() + (*vel)[1].m()*N.y());
    
    b[0] +=- 0.5 * DTSUB * DTSUB * ((*vel)[0].x() * (*oldphi)[0].m() + (*vel)[1].y()*(*oldphi)[0].m()) 
	* ((*vel)[0].m() * N.x() + (*vel)[1].m()*N.y());
      
      // equation not used
      b[1] += 0.0;
    }
  };

  // right hand side for h and a
  class TGRhs : public virtual DomainRightHandSide
  {
    SeaIceData data;
    
    mutable FemFunction *oldh, *vel;

    void SetFemData(FemData& q) const 
    { 
      assert(q.find("oldh")!=q.end()); 
      oldh = &q["oldh"];

      assert(q.find("vel")!=q.end()); 
      vel = &q["vel"];
    }

  public:
    
  TGRhs(const SeaIceData& dat) : DomainRightHandSide(), data(dat)
      {
      }
  TGRhs* createNew() const { return new TGRhs(data); } 

    std::string GetName() const {return "TG Rhs";}    
    int GetNcomp() const {return 2; }
    
    void operator()(VectorIterator b, const TestFunction& N, const Vertex2d& v) const 
    {
      // equation for H
      b[0] += (*oldh)[0].m() * N.m() 
	- DTSUB *( (*vel)[0].x() + (*vel)[1].y()) * (*oldh)[0].m() * N.m() 
	- DTSUB * ((*vel)[0].m() * (*oldh)[0].x() + (*vel)[1].m()*(*oldh)[0].y()) * N.m() 
	- 0.5 * DTSUB * DTSUB * ((*vel)[0].m() * (*oldh)[0].x() + (*vel)[1].m()*(*oldh)[0].y()) 
	* ((*vel)[0].m() * N.x() + (*vel)[1].m()*N.y());
    
    b[0] +=- 0.5 * DTSUB * DTSUB * ((*vel)[0].x() * (*oldh)[0].m() + (*vel)[1].y()*(*oldh)[0].m()) 
	* ((*vel)[0].m() * N.x() + (*vel)[1].m()*N.y());
      
      // equation for A
      b[1] += (*oldh)[1].m() * N.m() 
	- DTSUB * ((*vel)[0].x() + (*vel)[1].y()) * (*oldh)[1].m() * N.m() 
	- DTSUB * ((*vel)[0].m() * (*oldh)[1].x() + (*vel)[1].m()*(*oldh)[1].y()) * N.m() 
	- 0.5 * DTSUB * DTSUB * ((*vel)[0].m() * (*oldh)[1].x() + (*vel)[1].m()*(*oldh)[1].y()) 
	* ((*vel)[0].m() * N.x() + (*vel)[1].m()*N.y());
    
    b[1] +=- 0.5 * DTSUB * DTSUB * ((*vel)[0].x() * (*oldh)[1].m() + (*vel)[1].y()*(*oldh)[1].m()) 
	* ((*vel)[0].m() * N.x() + (*vel)[1].m()*N.y());
    }
  };


  // class for specifying problem's right hand side
  class MyRightHandSide : public DomainRightHandSide
  {
    SeaIceData data;
    
  public:

   
    void SetFemData(FemData& q) const 
    {
    }

    int GetNcomp()        const { return 4; }
    std::string GetName() const { return "Rhs"; }
    

  MyRightHandSide(const SeaIceData& dat) : DomainRightHandSide(), data(dat)
      {
	assert(data.rho>0);
	assert(data.rhoa>0);
	assert(data.Lref>0);
	assert(data.Tref>0);

      }

     MyRightHandSide* createNew() const { return new MyRightHandSide(data); }

    double operator()(int comp, const Vertex2d& v) const
    {
     
      
      double SZ = data.rhoa/data.rho * data.Cda * data.Lref;
      
      double time = TIME * data.Tref;
      double X = v.x()*data.Lref;
      double Y = v.y()*data.Lref;
    
      // --- divergent wind ----
      double ux = 0.0;
      double uy = 0.0;
      if (v.x() > 0.0025) {
          ux = 15/data.Lref*data.Tref; 
      } else if (v.x() < 0.0025) { 
          ux = -15/data.Lref*data.Tref; 
      }
      // --- divergent wind ----

      double vw    = sqrt(ux*ux + uy*uy);

      if (comp==0)
	return SZ * vw * (ux * cos(data.theta_a) - uy * sin(data.theta_a));
      else if (comp==1)
	return SZ * vw * (uy * cos(data.theta_a) + ux * sin(data.theta_a));
      else
	return 0;
   
    }
  };
  

  /*--------------------------------------------------*/
  
  
  class MyCI : public ComponentInformationBase 
  {
    
  public:
      
    virtual void      GetScalarName   (int i, std::string& s_name) 
    {
      if (i==0) s_name = "vx";
      if (i==1) s_name = "vy";
    }
    virtual const int GetNVectors     () const { return 1; }
    virtual void      GetVectorName   (int i, std::string& s_name) const { s_name = "V";}
    virtual void      GetVectorIndices(int i, std::array<int,3>& fa) const
    {
      fa[0]=0; fa[1]=1; fa[2]=-1;
    }
  };

  
  class MyCIWater : public ComponentInformationBase 
  {
    
  public:
      
    virtual void      GetScalarName   (int i, std::string& s_name) 
    {
      if (i==0) s_name = "p";
      if (i==1) s_name = "vx";
      if (i==2) s_name = "vy";
    }
    virtual const int GetNVectors     () const { return 1; }
    virtual void      GetVectorName   (int i, std::string& s_name) const { s_name = "V";}
    virtual void      GetVectorIndices(int i, std::array<int,3>& fa) const
    {
      fa[0]=1; fa[1]=2; fa[2]=-1;
    }
  };

  
  class MyCITr : public ComponentInformationBase 
  {
    
  public:
      
    virtual void      GetScalarName   (int i, std::string& s_name) 
    {
      if (i==0) s_name = "h";
      if (i==1) s_name = "A";
    }
    virtual const int GetNVectors     () const { return 0; }
  };

  // main class for defining the problem to solve
  class SeaIceProblem : public ProblemDescriptorBase
  {
    SeaIceData data;
  public:
    Application* NewRightHandSide() const
    {
      return new MyRightHandSide(data);
    }

    Equation *NewEquation() const
    {
      return new MyEquation(data);
    }
    
    
  
    std::string GetName() const {return "SeaIce Problem";}
    void BasicInit(const ParamFile& pf)
    {
      data.BasicInit(pf);
      
      // equation to solve
      GetEquationPointer()      = new MyEquation(data);

      GetDirichletDataPointer() = new SeaIceDirichletData(pf);
      GetComponentInformationPointer() = new MyCI;
      GetRightHandSidePointer()=new MyRightHandSide(data);
      // 
      ProblemDescriptorBase::BasicInit(pf);
    }
  };

  // main class for defining the problem to solve
  class TGProblem : public ProblemDescriptorBase
  {
  public:
    SeaIceData data;
    
    std::string GetName() const {return "TG Problem";}

    Application *NewRightHandSide() const 
    {
      return new TGRhs(data);
    }
    
    void BasicInit(const ParamFile& pf)
    {
      data.BasicInit(pf);
     
      GetEquationPointer()      = new TransportEquation(data);// Not in use !!!!!
      GetDirichletDataPointer() = new TGDirichletData(pf);
      GetRightHandSidePointer() = new TGRhs(data); 
	
      ProblemDescriptorBase::BasicInit(pf);

    }
  };

  // main class for phi
  class PhiProblem : public ProblemDescriptorBase
  {
  public:
    SeaIceData data;
    
    std::string GetName() const {return "Phi Problem";}

    Application *NewRightHandSide() const 
    {
      return new PhiRhs(data); 
    }
    
    void BasicInit(const ParamFile& pf)
    {
      data.BasicInit(pf);
     
      GetEquationPointer()      = new TransportEquation(data);// Not in use !!!!!
      GetDirichletDataPointer() = new PhiDirichletData(pf); 
      GetRightHandSidePointer() = new PhiRhs(data); 
	
      ProblemDescriptorBase::BasicInit(pf);

    }
  };
  
  class MyBM : public BoundaryManager
  {
    IntSet _nocolors;
  public:
    const IntSet& GetDirichletDataColors        () const
    { return _nocolors; }
  };
}

/*----------------------------   problem.h     ---------------------------*/
/* end of #ifndef __problem_H */
#endif
/*----------------------------   problem.h     ---------------------------*/
  
