import matplotlib.pylab as plt
import numpy as np


SMALL_SIZE = 16
MEDIUM_SIZE = 18
BIGGER_SIZE = 20

plt.rc('font', size=SMALL_SIZE)          # controls default text sizes
plt.rc('axes', titlesize=SMALL_SIZE)     # fontsize of the axes title
plt.rc('axes', labelsize=BIGGER_SIZE)    # fontsize of the x and y labels
plt.rc('xtick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=MEDIUM_SIZE)    # legend fontsize
plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title

def iteration_data(filename):
    
    # open file and create a string list with data from file
    with open(filename+"/iteration_numbers.txt", "r") as file:
        read_string_list = []
        for lines in file:
            lines = lines.rstrip('\n').split() 
            read_string_list.append(lines)

    # delete every second entry starting from the second
    # dgh solver, o solver
    del read_string_list[1::2]
    
    # transform string list into integer array
    length_s = len(read_string_list)
    data = np.zeros(length_s, dtype=object)
    for i in range(length_s):
        data[i] = np.asarray(read_string_list[i], dtype=object).astype(int)
    
    # create array for (non-)lin solver
    lin_solver    = np.zeros([len(data)])
    nonlin_solver = np.zeros([len(data)])
    
    for i in range(len(data)):
        nonlin_solver[i] = data[i][0] # nonlin solver: first entry in each array
        lin_solver[i]    = 0
    
    return nonlin_solver

def plot_iteration3(file1, file2, file3):
    plt.figure(figsize=(15,8), dpi=300)
    t = np.linspace(1,50,50)
    plt.plot(t, file1[:50], marker='o', label='standard Newton solver')
    plt.plot(t, file2[:50], marker='o', label='modified Newton solver', linestyle='dashed')
    plt.plot(t, file3[:50], marker='o', label='Picard solver', linestyle='dashed')
    plt.xlabel('time step')
    plt.ylabel('solver iterations')
    plt.legend(loc='best') 
    plt.grid(True)
    return 0

path3 = './Results_shear_param_'

standard = iteration_data(path3+'DDD1_10ms')
modified = iteration_data(path3+'DDD08_10ms')
picard = iteration_data(path3+'DDD0_10ms')
plot_iteration3(standard, modified, picard)

print('modified Newton', np.mean(modified))
print('standard Newton', np.mean(standard))
print('Picard', np.mean(picard))
print('difference standard-modified', -(np.mean(standard)-np.mean(modified)))
print('difference standard-modified in percent', -(np.mean(standard)-np.mean(modified))/np.mean(standard)*100)
print('difference picard-modified', -(np.mean(picard)-np.mean(modified)))
print('difference picard-modified in percent', -(np.mean(picard)-np.mean(modified))/np.mean(picard)*100)

plt.savefig('./fig06.pdf')