# -*- coding: utf-8 -*-
"""
Created on Wed Nov 10 17:19:13 2021

@author: Saskia Kahl
"""

# import libraries
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

# font size
SMALL_SIZE = 16
MEDIUM_SIZE = 18
BIGGER_SIZE = 20

plt.rc('font', size=SMALL_SIZE)          # controls default text sizes
plt.rc('axes', titlesize=SMALL_SIZE)     # fontsize of the axes title
plt.rc('axes', labelsize=BIGGER_SIZE)    # fontsize of the x and y labels
plt.rc('xtick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=MEDIUM_SIZE)    # legend fontsize
plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title

###############################################################
#
# load data
#
###############################################################

solver_newton = {'path': './Results_shear_param_DDD1_10ms', 'label': 'Newton'}
solver_picard = {'path': './Results_shear_param_DDD0_10ms', 'label': 'Picard'}
solver_modnew = {'path': './Results_shear_param_DDD08_10ms', 'label': 'modified Newton'}

###############################################################
#
# residuum plot
#
###############################################################

def load_residuum_data(path,rn):
    residuum = pd.read_csv(path+'/residuum'+str(rn)+'.txt', header=0, names=['r'])
    return residuum.r

def plot_settings_residuum(rn):
    fig = plt.figure(figsize=(15,8), dpi=300)
    plt.xlabel('solver iteration')
    plt.ylabel('residual '+str(rn))
    plt.grid(True)
    return fig

###############################################################

picard_r48          = load_residuum_data(solver_picard['path'],48)
newton_r48          = load_residuum_data(solver_newton['path'],48)
modified_newton_r48 = load_residuum_data(solver_modnew['path'],48)

fig7 = plot_settings_residuum('')

plt.plot(newton_r48/newton_r48[0], label='standard Newton solver')
plt.plot(modified_newton_r48/modified_newton_r48[0], label='modified Newton solver')
plt.plot(picard_r48/picard_r48[0], label='Picard solver')
plt.axhline(y=modified_newton_r48[32]/modified_newton_r48[0], color='dimgray')

plt.yscale('log')
plt.legend(loc='lower right')
plt.show()
fig7.savefig('./fig07_relative.pdf')
