#include "loop.h"
#include "gascoignemesh2d.h"
#include <time.h>
#include  "backup.h"
#include  "stdmultilevelsolver.h"
#include  "sparseblockmatrix.h"
#include  "fmatrixblock.h"
#include  "compose_name.h"
#include <algorithm>

#include "../lib/gascoignetransfer.h"

using namespace std;

#include "stopwatch.h"


extern ofstream ELLIPSE_OUT;

double TIME, DT, DTSUB, DELTAMIN,DTUL,starttime,dt_print;
extern int zahl;
extern double STEUERUNG_MU;

namespace Gascoigne
{
  extern Timer GlobalTimer;

  class VertexCompare
  {
  public:
    bool operator() (const Vertex2d& v1, const Vertex2d& v2) const
    {
      if (v1.x()<v2.x()-1.e-12)
	return true;
      if (v1.x()>v2.x()+1.e-12)
	return false;

      if (v1.y()<v2.y()-1.e-12)
	return true;

      return false;
    }
  };

  string Loop::PrecondCGMass(GlobalVector& u, GlobalVector& f, const TimePattern& TP, double s)
  {
    double global_tol = 1.e-12;
    double tol        = 1.e-8;
    int    maxiter    = 50;
    
    bool reached;
    int iter = 0;
    
    GlobalVector g(u.ncomp(),u.n());
    GlobalVector r(u.ncomp(),u.n());
    GlobalVector d(u.ncomp(),u.n());
    
    assert(u.ncomp()==g.ncomp());
    assert(u.n()==g.n());
    
    MM.PrepareJacobi(s);
    MM.vmult_time(f,u,TP,-s);
    MM.JacobiVector(u);
    MM.Jacobi(f);
    
    r.equ(1,f);
    d.equ(1,f);
    double Res = r*r;
    double FirstRes = Res;

    
    if (sqrt(Res)<global_tol)
      {
	reached = true;
      }
    else
      {
	reached = false;
      }
    
    while(!reached && iter<maxiter)
      {
	iter++;
	g.zero();
	MM.vmult_time_Jacobi(g,d,TP,s);
	double lambda = Res/(g*d);
	
	u.add(lambda,d);
	r.add(-lambda,g);
	
	Res = r*r;
	if (Res < tol*tol * FirstRes || sqrt(Res)<global_tol)
	  {
	    reached = true;
	  }
	double betacg = -(r*g)/(d*g);
	d.sequ(betacg,1.,r);
      }
    
    MM.Jacobi(u);

    if(iter==maxiter)
      {
	return "too many iterations";
      }
    else
      {
	return "converged";
      }
  }

  void Loop::AssembleMassMatrix()
  {
    SparseStructure SA;
    GetMultiLevelSolver()->GetSolver()->GetDiscretization()->Structure(&SA);
    MM.ReInit(&SA);
    MM.zero();
  
    GetMultiLevelSolver()->GetSolver()->GetDiscretization()->MassMatrix(MM);
  
    const ColumnDiagStencil* ST = dynamic_cast<const ColumnDiagStencil* >(MM.GetStencil());
    assert(ST);
  
    LMM.resize(ST->n());
    LMM.zero();
  
    for (int row = 0; row<ST->n(); ++row)
      {
	for (int pos = ST->start(row); pos<ST->stop(row); ++pos)
	  LMM[row] += MM.GetValue(pos);
	assert(LMM[row]>0);
      }
  }
 

  void Loop::SolveTransport(VectorInterface& h,
			    VectorInterface& div,
			    VectorInterface& vel,
			    VectorInterface& hl,
			    VectorInterface& hh,
			    VectorInterface& f)
  {
  

    GlobalTimer.start("T vor");

    // Righthandside
    GetMultiLevelSolver()->SetProblem("tg");
    GetMultiLevelSolver()->Zero(f);
    GetMultiLevelSolver()->GetSolver()->AddNodeVector("oldh",h); 
    GetMultiLevelSolver()->GetSolver()->AddNodeVector("vel",vel);
    GetMultiLevelSolver()->GetSolver()->Rhs(f);
    GetMultiLevelSolver()->GetSolver()->DeleteNodeVector("oldh");
    GetMultiLevelSolver()->GetSolver()->DeleteNodeVector("vel");
    
    GlobalTimer.stop("T vor");

    // Low-Order
    GlobalVector& F    = GetMultiLevelSolver()->GetSolver()->GetGV(f);
    GlobalVector& H    = GetMultiLevelSolver()->GetSolver()->GetGV(h);
    GlobalVector& UL  = GetMultiLevelSolver()->GetSolver()->GetGV(hl);
    GlobalVector& DIV  = GetMultiLevelSolver()->GetSolver()->GetGV(div);

#pragma omp parallel for
    for (int i=0;i<H.n();++i)
      for (int c=0;c<H.ncomp();++c)
	{
	  H(i,c) = F(i,c) / ( LMM[i] );
	}

    UL=H;

    // High-Order 
    GlobalVector& UH  = GetMultiLevelSolver()->GetSolver()->GetGV(hh);
    UH.zero();
    TimePattern TP(2,2); TP.zero(); TP(0,0)=1.0; TP(1,1) = 1.0;
    PrecondCGMass(UH, F, TP, 1.0);
 
    const ColumnDiagStencil* ST = dynamic_cast<const ColumnDiagStencil* >(MM.GetStencil());
    assert(ST);
  
    GlobalVector Rplus(H.ncomp(),H.n()), Rminus(H.ncomp(),H.n());
    
#pragma omp parallel
    {
      nvector<double> maxi(2),mini(2),meanmini(2), meanmaxi(2), Pminus(2), Pplus(2), Qminus(2), Qplus(2);
#pragma omp for 
      for (int row = 0; row<ST->n(); ++row)
	{
	  // max and min vof low-order, Pplus, Pminus
	  mini[0]=maxi[0] = H(row,0);
	  mini[1]=maxi[1] = H(row,1);
	  
	  meanmini[0]=meanmaxi[0] = 0.0;
	  meanmini[1]=meanmaxi[1] = 0.0;
	  Pminus=-1.e-10; Pplus = 1.e-10;
	  for (int pos = ST->start(row); pos<ST->stop(row); ++pos)
	    {
	      int col = ST->col(pos);
	      mini[0] = min(mini[0],H(col,0));  mini[1] = min(mini[1],H(col,1));
	      maxi[0] = max(maxi[0],H(col,0));  maxi[1] = max(maxi[1],H(col,1));
	      
	      meanmini[0]+= min(0.,H(col,0)); meanmini[1]+= min(0.,H(col,1));
	      meanmaxi[0]+= max(0.,H(col,0)); meanmaxi[1]+= max(0.,H(col,1));
	      
	      Pplus[0]  += max(0.0, MM.GetValue(pos) * (UH(row,0) - UH(col,0)));
	      Pminus[0] += min(0.0, MM.GetValue(pos) * (UH(row,0) - UH(col,0)));
	      
	      Pplus[1]  += max(0.0, MM.GetValue(pos) * (UH(row,1) - UH(col,1)));
	      Pminus[1] += min(0.0, MM.GetValue(pos) * (UH(row,1) - UH(col,1)));
	    }

	  // Q + R
	  Qplus[0]  = LMM[row] * (maxi[0]-H(row,0));
	  Qplus[1]  = LMM[row] * (maxi[1]-H(row,1));
	  Qminus[0] = LMM[row] * (mini[0]-H(row,0));
	  Qminus[1] = LMM[row] * (mini[1]-H(row,1));
	  
	  Rplus(row,0) = min(1.0, Qplus[0]/Pplus[0]);
	  Rplus(row,1) = min(1.0, Qplus[1]/Pplus[1]);
	  Rminus(row,0) = min(1.0, Qminus[0]/Pminus[0]);
	  Rminus(row,1) = min(1.0, Qminus[1]/Pminus[1]);
	}
    }
    
    // correction
#pragma omp parallel for 
    for (int row = 0; row<ST->n(); ++row)
      {
	for (int pos = ST->start(row); pos<ST->stop(row); ++pos)
	  {
	    int col = ST->col(pos);
	    double fij0 = MM.GetValue(pos)*(UH(row,0)-UH(col,0));
	    double fij1 = MM.GetValue(pos)*(UH(row,1)-UH(col,1));
	    // Prelimiting
	    if(fij0*(UL(col,0)-UL(row,0))>0)
	      {
		fij0=0;
	      }
	    if(fij1*(UL(col,1)-UL(row,1))>0)
	      { 
		fij1=0;
	      }
	  
	    if (fij0 >= 0) 
	      {
		H(row,0) += 1.0/LMM[row] * min(Rplus(row,0), Rminus(col,0)) * fij0;
	      }
	    else 
	   
	      H(row,0) += 1.0/LMM[row] * min(Rminus(row,0), Rplus(col,0)) * fij0;

	    if (fij1 >=0) 
	      H(row,1) += 1.0/LMM[row] * min(Rplus(row,1), Rminus(col,1)) * fij1;
	    else 
	      H(row,1) += 1.0/LMM[row] * min(Rminus(row,1), Rplus(col,1)) * fij1;
		
	  }
      }
  }


  void Loop::run(const std::string& problemlabel)
  {

    GlobalTimer.start("alles");
    GlobalTimer.start("-> Vorbereitung");
      

    zahl=1;
    double tref;
    double dtmax, endtime;
    int prerefine;
    string _reloadu,_reloadh,_reloadphi,_reloadoldu;
    if (1)
      {
	DataFormatHandler DFH;
	DFH.insert("dt",   &DT, 0.);
	DFH.insert("dtmax",   &dtmax, 0.);
	DFH.insert("time", &starttime, 0.);
	DFH.insert("endtime", &endtime, 0.);
	DFH.insert("reloadu", &_reloadu);
	DFH.insert("reloadh", &_reloadh);
    DFH.insert("reloadphi", &_reloadphi);
	DFH.insert("reloadoldu", &_reloadoldu);
	FileScanner FS(DFH);
	FS.NoComplain();

	FS.readfile(_paramfile, "Loop");
	assert(DT>0.0);
	cout<<starttime<<"start"<<endl; }
    if (1)
      {
	DataFormatHandler DFH;
	DFH.insert("prerefine",   &prerefine, 0);
	FileScanner FS(DFH);
	FS.NoComplain();
	FS.readfile(_paramfile, "Mesh");
      }
    if (1)
      {
	DataFormatHandler DFH;
	DFH.insert("deltamin",   &DELTAMIN, 0.);
	DFH.insert("Tref", &tref, 0.);
	FileScanner FS(DFH);
	FS.NoComplain();
	FS.readfile(_paramfile, "Equation");
	assert(DELTAMIN>0.0);
	assert(tref>0.0);
      }
    string discname;
    if (1)
      {
	DataFormatHandler DFH;
	DFH.insert("discname",   &discname);
	FileScanner FS(DFH);
	FS.NoComplain();
	FS.readfile(_paramfile, "Solver");
	assert(discname=="CGQ1" || discname == "CGQ2");
      }

  
    // vectors for solution and right hand side
    VectorInterface u("u"), f("f"), oldu("oldu");
    VectorInterface h("h"),hl("hl"), hh("hh"), extu("extu"), div("div"),oldh("oldh"),oldoldu("oldoldu");
    
    VectorInterface phi("phi");
    VectorInterface phil("phil"), phih("phih"), oldphi("oldphi");
	VectorInterface phif("phif");
    Matrix A("A");
    
    VectorInterface DGH("DGH","cell");
    VectorInterface INDI("INDI","cell");
    
    PrintMeshInformation();
    // initialize problem, solver and vectors 
    GetMultiLevelSolver()->ReInit();
    GetMultiLevelSolver()->SetProblem("seaice");
    GetMultiLevelSolver()->ReInitMatrix(A);
    GetMultiLevelSolver()->ReInitVector(u);
    GetMultiLevelSolver()->ReInitVector(oldu);
    GetMultiLevelSolver()->ReInitVector(f); 

    GetMultiLevelSolver()->ReInitVector(h);
    GetMultiLevelSolver()->ReInitVector(hl);
    GetMultiLevelSolver()->ReInitVector(hh);
    GetMultiLevelSolver()->ReInitVector(extu);
    GetMultiLevelSolver()->ReInitVector(div);
    GetMultiLevelSolver()->ReInitVector(oldh);
    GetMultiLevelSolver()->ReInitVector(oldoldu);
    
    GetMultiLevelSolver()->ReInitVector(phi);
    GetMultiLevelSolver()->ReInitVector(phil);
    GetMultiLevelSolver()->ReInitVector(phih);
    GetMultiLevelSolver()->ReInitVector(oldphi);
	GetMultiLevelSolver()->ReInitVector(phif);
    
    GetMultiLevelSolver()->ReInitVector(DGH);
    GetMultiLevelSolver()->ReInitVector(INDI);
    
    GetSolverInfos()->GetNLInfo().control().matrixmustbebuild() = 1;
    GetMultiLevelSolver()->GetSolver()->OutputSettings();

    cout << "------------------------------" << endl;
    cout << "sea-ice " << endl;
    cout << "------------------------------" << endl << endl;
  
    // Sea-ice
    GetMultiLevelSolver()->SetProblem("seaice");
    if (_initial=="reload")	
      {
	abort();
	GetMultiLevelSolver()->GetSolver()->Read(u,_reloadu);
	GetMultiLevelSolver()->GetSolver()->Read(oldu,_reloadoldu);   
      }
    else 
      {
	InitSolution(u);     
	GetMultiLevelSolver()->Equ(oldu,1.0,u);
      }

    // TG Transport
    GetMultiLevelSolver()->SetProblem("tg");
    if (_initial=="reload")
      { GetMultiLevelSolver()->GetSolver()->Read(h,_reloadh);
      }
    else 
      InitSolution(h);
    
    // TG Transport phi
    GetMultiLevelSolver()->SetProblem("phi tg");
    if(_initial=="reload") {
        GetMultiLevelSolver()->GetSolver()->Read(phi,_reloadphi);
    } else {
        InitSolution(phi);
    }
    
    
    GetMultiLevelSolver()->Equ(hl,1.0,h);
    GetMultiLevelSolver()->Equ(phil,1.0,phi); 

    // mass matrix:
    AssembleMassMatrix();
 
	
    double stepback=0.0;
    double writenext= 0;
    int writeiter = 0;
    string res;
  
    nvector<double> functionals;
	
  
    int timeinc=0;
  
    clock_t start, end;
    double cpu_time_used,a;

    TIME=starttime;

    nvector<double>      Jtotal,Jtotal1;

    GlobalTimer.stop("-> Vorbereitung");
    for (_iter=1; _iter<=_niter; _iter++)
      {
	GlobalTimer.start("-> Iteration");
	  

	TIME += DT;

	GetMultiLevelSolver()->GetSolver()->Equ(oldu,1.0, u);
	GetMultiLevelSolver()->GetSolver()->Equ(extu,1.0, u);
	
	GetSolverInfos()->GetNLInfo().control().matrixmustbebuild() = 1;

	cout << "\n time step " << _iter << " " 
	     << "\t" << TIME*tref/60/60/24 << " days" << endl;

	
	//--------------------transport flux-corrected (TG)--------------------//
	
	// Transportequation
	GlobalTimer.start("--> Transport");
	
	GetMultiLevelSolver()->GetSolver()->Equ(oldh,1.0,hl); 
    GetMultiLevelSolver()->GetSolver()->Equ(oldphi,1.0,phil); 
	
	int NSUB = 1; // subcycle

	for (int sub=0; sub<NSUB; ++sub)
	  {

	    DTSUB = DT/NSUB;
	    std::cout << GetMultiLevelSolver()->GetSolver()->GetGV(u).norm() << std::endl;

	    SolveTransport(h, div, u, hl, hh, f);
        SolveTransport(phi, div, u, phil, phih, f); 

	    GlobalVector& H = GetMultiLevelSolver()->GetSolver()->GetGV(h);
#pragma omp parallel for 
	    for (int i=0; i<H.n(); ++i) 
	      {
		H(i,1) = min(1.0, H(i,1));
		H(i,1) = max(0.0, H(i,1));
		H(i,0) = max(0.0, H(i,0));
	      }
	    GlobalVector& UL = GetMultiLevelSolver()->GetSolver()->GetGV(hl);
#pragma omp parallel for
	    for (int i=0; i<UL.n(); ++i)
	      {
		UL(i,1) = min(1.0, UL(i,1));
		UL(i,1) = max(0.0, UL(i,1));
		UL(i,0) = max(0.0, UL(i,0));
	      }
        
	  }
	
	
	GetMultiLevelSolver()->GetSolver()->Visu("Results/h", h, _iter);
	GetMultiLevelSolver()->GetSolver()->Visu("Results/v",u,_iter); 
    GetMultiLevelSolver()->GetSolver()->Visu("Results/phi", phi, _iter);
	GlobalTimer.stop("--> Transport");
  
	// set sea ice problem (to solve sea ice momentum equation)
	cout<<"Momentum"<<endl;
	GlobalTimer.start("--> Momentun eq"); 
	   GetMultiLevelSolver()->SetProblem("seaice");
	   GetSolverInfos()->GetNLInfo().control().matrixmustbebuild() = 1;
	   GetMultiLevelSolver()->AddNodeVector("oldu", oldu);
	   GetMultiLevelSolver()->AddNodeVector("extu", extu);
	   GetMultiLevelSolver()->AddNodeVector("H", hl);
       GetMultiLevelSolver()->AddNodeVector("PHI", phil);
	   GetMultiLevelSolver()->GetSolver()->AddCellVector("DGH", DGH); 
	   GetMultiLevelSolver()->GetSolver()->AddCellVector("INDI", INDI);    
	   GetMultiLevelSolver()->GetSolver()->Equ(oldu,1.0, u);
	   GetMultiLevelSolver()->GetSolver()->Equ(oldoldu, 1.0, oldu);
	   GetMultiLevelSolver()->AssembleMatrix(A,u);
	   GetMultiLevelSolver()->ComputeIlu(A,u); 
      
	   res=Solve(A,u,f,"Results/u");
	   
	   GetMultiLevelSolver()->GetSolver()->Visu("Results/v",u,_iter);

	   functionals = Functionals(u,f);

	   GetMultiLevelSolver()->DeleteNodeVector("oldu");
	   GetMultiLevelSolver()->DeleteNodeVector("extu");
	   GetMultiLevelSolver()->DeleteNodeVector("H");
       GetMultiLevelSolver()->DeleteNodeVector("PHI");
	   GetMultiLevelSolver()->GetSolver()->DeleteCellVector("DGH");
	   GetMultiLevelSolver()->GetSolver()->DeleteCellVector("INDI");
	   GlobalTimer.stop("--> Momentun eq");
      
	stringstream str;
	str << "fv_bench_kinetic.txt";
	ofstream OUTF(str.str().c_str(),ios::app);
	OUTF.precision(10);
	OUTF << TIME << " " << functionals<<endl;
	
    
	writenext+=DT;
      	   

	GlobalTimer.stop("-> Iteration"); 
	if (TIME >= endtime*24*60*60/tref) break;
	  
      }
	
    GlobalTimer.stop("alles");
  
    GlobalTimer.print100();
  }



}






 
