#include  "problem.h"
#include  "loop.h"
#include "domainfunctional.h"
#include  "filescanner.h"
#include  "stdloop.h"
#include "gascoignemesh2d.h"
#include  "stdmultilevelsolver.h"
#include  "stdsolver.h"
#include  "simplematrix.h"
#include  "domainfunctional.h"
#include  "residualfunctional.h" 
#include  "dirichletdatabycolor.h"
using namespace Gascoigne;
using namespace std;

extern double DELTAMIN;

class Force1 : public virtual ResidualFunctional
{
  
public:
  DataFormatHandler DFH;

  Force1() : ResidualFunctional() 
  {
    __comps.push_back(0);
    __cols.insert(3);
    __scales.push_back(1.0);
    
    //ExactValue() = 0.;
    __DD  = new DirichletDataByColor(GetComps(),GetColors(),GetScales());
  }

  std::string GetName() const { 
    return "Force 1";
  }
};

class Force2 : public virtual ResidualFunctional
{
public:
  Force2() : ResidualFunctional() 
  {
    __comps.push_back(1);
    __cols.insert(1);
    __scales.push_back(1.0);
    
    ExactValue() = 0.;
    __DD  = new DirichletDataByColor(GetComps(),GetColors(),GetScales());
  }

  std::string GetName() const { 
    return "Force 2";
  }
};



int main(int argc, char** argv)
{
  // parameter file as argument
  ParamFile paramfile("gascoigne.param");
  if(argc>=2) 
    paramfile.SetName(argv[1]);

  // define problem
  // sea ice mometum equation
  SeaIceProblem LPD;
  LPD.BasicInit(paramfile);
  // transport of h and A
  TGProblem TGPD;
  TGPD.BasicInit(paramfile);

  // transport of phi
  PhiProblem PPD;
  PPD.BasicInit(paramfile);

  ProblemContainer PC;
  PC.AddProblem("seaice", &LPD);
  PC.AddProblem("tg",    &TGPD);
  PC.AddProblem("phi tg", &PPD);

  // output functionals 
  FunctionalContainer FC;
  Force1 F1;
  Force2 F2;

  FC.AddFunctional("1 Force1", &F1);
  FC.AddFunctional("2 Force2", &F2);

  // loop for program control
  Loop loop;
  loop.BasicInit(paramfile, &PC, &FC);

  // start solving
  loop.run("seaice");

  return 0;
}
