#include  "myequation.h"
#include  "filescanner.h"
#include "algorithm"


extern double TIME;
extern double DT,DDD;
extern double DELTAMIN;
extern double STEUERUNG_MU,zahl;


/*-----------------------------------------*/

namespace Gascoigne
{
  void SeaIceData::BasicInit(const ParamFile& paramfile) 
  {
    DataFormatHandler DFH;
    DFH.insert("v_in", &vin0, 0.);
    DFH.insert("rho", &rho, 0.);
    DFH.insert("rhow", &rhow, 0.);
    DFH.insert("Tref",&Tref,0.0);
    DFH.insert("Lref",&Lref,0.0);
    DFH.insert("Pstern",&Pstern,2.75e4);
    DFH.insert("ellipse",&ellipse,2.0);
    DFH.insert("C",&C,20.0);
    DFH.insert("Cdw",&Cdw,5.2e-3);
    DFH.insert("f",&f,0.0);
    DFH.insert("theta_w",&theta_w,0.0);
    
    DFH.insert("rhoa", &rhoa, 0.);
    DFH.insert("Cda",&Cda,0.0);
    DFH.insert("theta_a",&theta_a,0.0);
    DFH.insert("windx",&windx,0.0);
    DFH.insert("windy",&windy,0.0);

    FileScanner FS(DFH);
    FS.NoComplain();
    FS.readfile(paramfile, "Equation");
    assert(rho>0);
    assert(Tref>0);
    assert(Lref>0);
    assert(Pstern>0);
    assert(ellipse>0);
    assert(C>0);
    //assert(f>0);

    MZ = 0.5*Tref*Tref * Pstern / rho / Lref / Lref;
    std::cout << "Mehlmann-Zahl " << MZ << std::endl;
  }  

  // sea ice mometum equation 
  void MyEquation::point(double h, const FemFunction &U, const Vertex2d &v) const
  {
    h_=h;

    uwx= 0.0*data.Tref/data.Lref;
    uwy= 0.0*data.Tref/data.Lref;
  
  }
 

  
  void MyEquation::Form(VectorIterator b, const FemFunction& U, const TestFunction& N) const
  {
    // ocean drag tau_w
    // wind drag is caculated in problem.h 
    double WZ = data.rhow/data.rho * data.Cdw * data.Lref;
    
    double vd_x = U[0].m() - uwx;
    double vd_y = U[1].m() - uwy;
    
    double vd = sqrt(vd_x*vd_x+vd_y*vd_y+1.e-8);

      
    b[0] += WZ * vd *(vd_x * cos(data.theta_w) - vd_y*sin(data.theta_w)) * N.m();
    b[1] += WZ * vd *(vd_y * cos(data.theta_w) + vd_x*sin(data.theta_w)) * N.m();


    double dmin = DELTAMIN*data.Tref;
    
    // DELTA of the viscosity 

    double DELTAsquare =
      (1.0 + pow(data.ellipse,-2.0)) * (U[0].x()*U[0].x() + U[1].y()*U[1].y())
      + pow(data.ellipse,-2.0) * pow(U[0].y() + U[1].x(),2.0)
      + 2.0 * (1.0-pow(data.ellipse,-2.0)) * U[0].x()*U[1].y();
    
    DELTAsquare +=   dmin*dmin;
    
     double DELTA = sqrt(DELTAsquare);

  //---------------Flux-----------------------------------------
    
    // Coriolis
    b[0] += -data.Tref*(*H)[0].m()*data.f*(vd_y)*N.m();
    b[1] +=  data.Tref*(*H)[0].m()*data.f*(vd_x)* N.m();
    
    // time derivative
    b[0] += (*H)[0].m()*(U[0].m() - (*oldu)[0].m()) / DT * N.m();
    b[1] += (*H)[0].m()*(U[1].m() - (*oldu)[1].m()) / DT * N.m();

    double ef = exp(-data.C*(1.0-(*H)[1].m()));


    ///////  (sigma, nabla phi)	
    // nabla u, nabla phi
    // nable u^T, nabla phi
    for (int i=0;i<2;++i)
      for (int j=0;j<2;++j)
	{
      if ( (*PHI)[0].m() > 0.3){ 
        b[i] += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * (ef+1) / DELTA * ( U[i][j+1] + U[j][i+1] ) * N[j+1];
	      b[i] += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * (ef+1) / DELTA * U[j][j+1]*N[i+1];
      } else {
        b[i] += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * ef / DELTA * ( U[i][j+1] + U[j][i+1] ) * N[j+1];
	      b[i] += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * ef / DELTA * U[j][j+1]*N[i+1];
      }
	}

   // ice strength P
   if( (*PHI)[0].m() > 0.3){
    b[0] += -data.MZ * (*H)[0].m() * (ef-1) * N.x();
    b[1] += -data.MZ * (*H)[0].m() * (ef-1) *N.y();
   } else {
    b[0] += -data.MZ * (*H)[0].m() * ef * N.x();
    b[1] += -data.MZ * (*H)[0].m() * ef *N.y();
   }
    
    
    
  }
  
  
  // sea ice mometum equation Matrix
  void MyEquation::Matrix(EntryMatrix& A, const FemFunction& U, const TestFunction& M, const TestFunction& N) const
  {

    //  tau_w ocean drag 
    double WZ = data.rhow/data.rho * data.Cdw * data.Lref;
   
    double vd_x = U[0].m() - uwx;
    double vd_y = U[1].m() - uwy;
  
    double vd = sqrt(vd_x*vd_x+vd_y*vd_y+1.e-8);
    double vd_0 = 1.0/(2.0*vd) * 2.0 * vd_x * M.m();
    double vd_1 = 1.0/(2.0*vd) * 2.0 * vd_y * M.m();
 
    //b[0] += WZ * vd   *(vd_x * cos(theta_w) - vd_y*sin(theta_w) ) * N.m();
    A(0,0) += WZ * vd_0 *(vd_x * cos(data.theta_w) - vd_y*sin(data.theta_w) ) * N.m();
    A(0,1) += WZ * vd_1 *(vd_x * cos(data.theta_w) - vd_y*sin(data.theta_w) ) * N.m();
    A(0,0) += WZ * vd   *(M.m() * cos(data.theta_w)                    ) * N.m();
    A(0,1) += WZ * vd   *(                    -M.m()*sin(data.theta_w) ) * N.m();
  
    //b[1] += WZ * vd   * (vd_y * cos(theta_w) + vd_x*sin(theta_w)) * N.m();
    A(1,0) += WZ * vd_0 * (vd_y * cos(data.theta_w) + vd_x*sin(data.theta_w)) * N.m();
    A(1,1) += WZ * vd_1 * (vd_y * cos(data.theta_w) + vd_x*sin(data.theta_w)) * N.m();
    A(1,1) += WZ * vd   * (M.m() * cos(data.theta_w)                   ) * N.m();
    A(1,0) += WZ * vd   * (                     M.m()*sin(data.theta_w)) * N.m();

    
    //derivative of Delta
    double dmin = DELTAMIN*data.Tref;
    double DELTAsquare =
      (1.0 + pow(data.ellipse,-2.0)) * (U[0].x()*U[0].x() + U[1].y()*U[1].y())
      + pow(data.ellipse,-2.0) * pow(U[0].y() + U[1].x(),2.0)
      + 2.0 * (1.0-pow(data.ellipse,-2.0)) * U[0].x()*U[1].y();


    DELTAsquare +=  dmin*dmin;      
    double DELTA = sqrt(DELTAsquare);
    
    // derivativ w.r.t Delta
    double DELTAsquare_0 = 
      (1.0 + pow(data.ellipse,-2.0)) * (2.0*U[0].x()*M.x())
      + pow(data.ellipse,-2.0) * 2.0 * (U[0].y() + U[1].x()) * M.y()
      + 2.0 * (1.0-pow(data.ellipse,-2.0)) * M.x()*U[1].y();
    double DELTAsquare_1 = 
      (1.0 + pow(data.ellipse,-2.0)) * (2.0 * U[1].y()*M.y())
      + pow(data.ellipse,-2.0) * 2.0*(U[0].y() + U[1].x()) * M.x()
      + 2.0 * (1.0-pow(data.ellipse,-2.0)) * U[0].x()*M.y();

     double DELTA_0 = -0.5 * pow(DELTAsquare,-1.5) * DELTAsquare_0*DDD;
     double DELTA_1 = -0.5 * pow(DELTAsquare,-1.5) * DELTAsquare_1*DDD;

    
    //b[0] += -data.Tref*(*H)[0].m()*f*(vd_y)*N.m();
    A(0,1) += -data.Tref*(*H)[0].m()*data.f*M.m() * N.m();

    //b[1] +=  data.Tref*(*H)[0].m()*f*(vd_x)* N.m();
    A(1,0) +=  data.Tref*(*H)[0].m()*data.f*M.m() * N.m();
    
    // time derivative
    A(0,0) +=  (*H)[0].m()* M.m() / DT * N.m();
    A(1,1) +=  (*H)[0].m()* M.m() / DT * N.m();

    double ef   = exp(-data.C*(1.0-(*H)[1].m()));
     
    // nabla u, nabla phi
    // nable u^T, nabla phi
    for (int i=0;i<2;++i)
      for (int j=0;j<2;++j)
	{
    if( (*PHI)[0].m() > 0.3){
      //	  b[i] += MZ * (*H)[0].m() * pow(ellipse,-2.0) * ef / DELTA * ( U[i][j+1] + U[j][i+1] ) * N[j+1];
	  A(i,0) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * (ef+1) * DELTA_0 * ( U[i][j+1] + U[j][i+1] ) * N[j+1];
          A(i,1) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * (ef+1) * DELTA_1 * ( U[i][j+1] + U[j][i+1] ) * N[j+1];

          A(i,i) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * (ef+1) / DELTA * M[j+1] * N[j+1];
	  A(i,j) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * (ef+1) / DELTA * M[i+1] * N[j+1];


	  A(i,0) += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * (ef+1) * DELTA_0 * U[j][j+1]*N[i+1];
	  A(i,1) += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * (ef+1) * DELTA_1 * U[j][j+1]*N[i+1];
	  
	  // // b[i] += MZ * (*H)[0].m() * (1.0-pow(ellipse,-2.0)) * ef / DELTA * U[j][j+1]*N[i+1];
	  A(i,j) += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * (ef+1) / DELTA * M[j+1]*N[i+1];  
    } else {
      //	  b[i] += MZ * (*H)[0].m() * pow(ellipse,-2.0) * ef / DELTA * ( U[i][j+1] + U[j][i+1] ) * N[j+1];
	  A(i,0) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * ef * DELTA_0 * ( U[i][j+1] + U[j][i+1] ) * N[j+1];
          A(i,1) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * ef * DELTA_1 * ( U[i][j+1] + U[j][i+1] ) * N[j+1];

          A(i,i) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * ef / DELTA * M[j+1] * N[j+1];
	  A(i,j) += data.MZ * (*H)[0].m() * pow(data.ellipse,-2.0) * ef / DELTA * M[i+1] * N[j+1];


	  A(i,0) += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * ef * DELTA_0 * U[j][j+1]*N[i+1];
	  A(i,1) += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * ef * DELTA_1 * U[j][j+1]*N[i+1];
	  
	  // // b[i] += MZ * (*H)[0].m() * (1.0-pow(ellipse,-2.0)) * ef / DELTA * U[j][j+1]*N[i+1];
	  A(i,j) += data.MZ * (*H)[0].m() * (1.0-pow(data.ellipse,-2.0)) * ef / DELTA * M[j+1]*N[i+1];
    }
	  
	}


    
  }


}
