#include "solver.h"
#include <iostream>
#include <fstream>

int zahl, it;
double DDD=1.0;
int writeiter;

using namespace std;

namespace Gascoigne{
double MyMLS::NewtonUpdate(double& rr, VectorInterface& x, VectorInterface& dx, VectorInterface& r, const VectorInterface& f, NLInfo& nlinfo)
{
  const CGInfo& linfo = nlinfo.GetLinearInfo();
  bool lex  = linfo.control().status()=="exploded";

  double nn = NewtonNorm(dx);

  double nnl2=GetSolver()->GetGV(dx).norm();

  double nr = GetSolver(ComputeLevel)->Norm(r);

  ostringstream ss;
  ss << "mat_nn_it_DD_neu1" << it << ".dat";
  string datname = ss.str();

  if (nn>1.e30)  lex =1;
  if (!(nn>=0.)) lex =1;
  if (nr>1.e30)  lex =1;
  if (!(nr>=0.)) lex =1;

  if(lex)
    {
      nlinfo.control().status()="diverged";
      cerr << "linear : " << linfo.control().status() << endl;
      cerr << "nonlinear : " << nn << endl;
      return NewtonNorm(dx);
    }

  double omega = 0.60;
  double relax = 1.0;

  
  GetSolver(ComputeLevel)->SetPeriodicVectorZero(dx);
 
  GetSolver(ComputeLevel)->Add(x,relax,dx);
  NewtonResidual(r,x,f);
  rr = NewtonNorm(r);

  string message = "";
  int diter=0;
  

  
  for(diter=0;diter<nlinfo.user().maxrelax();diter++)
    {
      message = nlinfo.check_damping(diter,rr);
       
	
      if (message=="ok")       break;
      if (message=="continue") 
      {
        GetSolver(ComputeLevel)->Add(x,relax*(omega-1.),dx);  

        NewtonResidual(r,x,f);
        rr = NewtonNorm(r);
        relax *= omega;
        continue;
      }
      if (message=="exploded")
      {
        GetSolver(ComputeLevel)->Add(x,-relax,dx);
        relax = 0.;
        cout << "Damping exploded !!!!!" << endl;
        nlinfo.control().status() = "diverged";
        break;
      }
    }
  
  nlinfo.check_damping(diter,rr);

  return NewtonNorm(dx);
}


  void MyMLS::newton(Matrix& A, VectorInterface& u, const VectorInterface& f, VectorInterface& r, VectorInterface& w, NLInfo& info)
{
  DDD=1.0;

  double rho1=0.0;
  double rho2=0.0;
  double ET1=1.0;
  double ET2=1.0;
  info.reset();
  double rr = NewtonResidual(r,u,f);
  bool reached = info.check(0,rr,0.);
  NewtonOutput(info);
  NewtonPreProcess(u,f,info);
  nvector<int> nlin,nresi;
  double res_old,res_new,res_mat;
  
  it=0.0;
  
  std::vector<double> ressi(200);
  ressi[0]=1.0;

 

  for(it=1; !reached; it++)
    {
      NewtonMatrixControl(A,u,info);
      NewtonVectorZero(w);
      
      NewtonLinearSolve(A,w,r,info.GetLinearInfo());
      nlin.push_back(info.GetLinearInfo().control().iteration());
   
      double rw = NewtonUpdate(rr,u,w,r,f,info);
      
      ressi[it]=rr;
      reached = info.check(it,rr,rw);
     
      rho1 = info.statistics().lastrate();

      NewtonOutput(info);
      
      
      nresi.push_back(rr);

      ET1=DDD;
      
      DDD=std::min(ET1*(0.2+4/(0.7+exp(1.51*rho1))),1.);

	 if(DDD<0.80)
	 DDD=1;
       	 cout  <<  "Steuerung Newton rho/DDD/ET: " << rho1 << "\t" << DDD << "\t"<< rr << endl; 
       
       
    }

  // create file to write (non-)linear solver iteration numbers
  ofstream file;
  file.open("iteration_numbers.txt",ios::app);
  file << it-1 << " " << nlin << endl;
  file.close();
  
  NewtonPostProcess(u,f,info);
}
  
}
